class Pask::Event
  CALL        = "call"
  C_CALL      = "c-call"
  RETURN      = "return"
  C_RETURN    = "c-return"
  KNOWN_NAMES = [CALL, C_CALL, RETURN, C_RETURN].map!(&:freeze)

  def initialize(name, other)
    @name = name
    @other = other
    @created_at = Time.now
    @signature = nil
    @to_f = nil
    @emit_by = binding.eval "self"
  end

  def emit_by
    # FIXME: messed up for C calls and returns. maybe return nil for them?
    @emit_by
  end

  def to_f
    @to_f ||= @created_at.to_f
  end

  def created_at
    @created_at
  end

  def file
    @other[:file]
  end

  def lineno
    @other[:lineno]
  end

  def emit_by_method
    @other[:method]
  end

  def binding
    @other[:binding]
  end

  def __binding__
    Kernel.binding
  end

  def any_call?
    call? or c_call?
  end

  def return?
    @name == RETURN
  end

  def c_return?
    @name == C_RETURN
  end

  def call?
    @name == CALL
  end

  def c_call?
    @name == C_CALL
  end

  def any_return?
    return? or c_return?
  end

  def origin?(mod)
    if emit_by.respond_to?(:ancestors)
      emit_by.ancestors.include?(mod)
    else
      mod === emit_by
    end
  end

  def signature
    @signature ||= if emit_by.kind_of?(Module)
      [emit_by, @method].join(".")
    else
      [emit_by.class, @method].join("#")
    end
  end

  def -(other)
    unless other.respond_to?(:to_f)
      raise TypeError, "cannot coerce argument to a float"
    end
    to_f - other.to_f
  end
end
